function [sams,nChans,sampleSize,sRate,nLeft,dt] = ...
                                      haruIn_CF2(filename, start, nframe, chans)
%haruIn   Read data from a 3rd generation Haru-format file with CF2 
%   - this accompanied a harufile format change in 2013/2014
%
% [sams,nchan,sampleSize,sRate,nLeft,dt] = haruIn(filename, start, n, chans)
%    Read Haru Matsumoto's 2nd generation file format.  Such files
%    typically have names like 00000318.DAT or DATAFILE.318, or possibly
%    r0125502.28 or r0125502.28m130.
%    
%    Input arguments:
%      start      is the starting sample number; default is 0, meaning the
%                 first sample in the file
%      nframe     is the number of samples to read per channel; default is Inf,
%                 meaning read the entire file
%      chans      says which channels to read; default is all of them; channel
%                 numbering starts at 1
%
%    Output arguments:
%      sams       is a column vector of samples (or multi-column array, if
%                 chans's length is >= 2)
%      nchan      says how many channels are in the file (equals 1 for all
%                 extant Haru-format files, but who knows about the future)
%      sampleSize is the number of bytes per sample, 1 or 2
%      sRate      is the sampling rate, Hz
%      nLeft      is the number of sample frames remaining in the file after
%                 reading (use haruIn(filename, 0, 0) to get the total number
%                 of samples in the file)
%      dt         is the date/time (in GMT) that the file starts, encoded as
%                 in datenum
%
% Some Haru-files (ETP before Nov. 99, MAR) do not encode their sampling rate,
% so I have to guess.  I'll tell you when I'm guessing.  To override this 
% guess, do something like this:
%          global opHaruSrate; opHaruSrate=112;
%
% 
%
% Dave Mellinger
% 24 Nov 2003

% There are two time-date strings in Haru files.  The first is from the QTech
% clock, which is generally the more accurate, temperature-corrected one.  
% The second is from the Real-Time Clock (RTC), essentially the computer's
% system clock; it's less accurate.
%
% The RTC is set periodically (Matt said maybe once a day) from the QTech
% clock, so the two should never be far apart.  However, upon a reboot, the
% QTech clock doesn't know what time it is, so it gets set from the RTC.  
% So the QTech clock can have a small quantum jump at that point, equal to
% the amount that the RTC was off.  One good way to estimate the jump is
% by looking at the last Haru-file recorded before the reboot, and calculate
% the difference between the two clocks.  A better way would probably be to
% estimate the drift rate between the two clocks, then add the appropriate
% amount of drift to that last-file clock difference.  I've never done the
% latter.
%
fd = fopen(filename, 'r', 'b');   % read-only; big-endian byte order
%
% Get the date string, parse into dt.
[fs,yr,jd,hr,mn,sc,samprate,t,y]=readCF2hdr(filename,0);
%
dt = fs;    % time of start of file
sRate = samprate;
sampleSize = 2;
nChans = 1;
precision = 32768;
%
headerLen = 256;
%
fseek(fd, 0, 'eof');
nFramesTotal = (ftell(fd) - headerLen) / nChans / sampleSize;
fseek(fd, headerLen, 'bof');

if (nargin < 2), start = 0;        end
if (nargin < 3), nframe = inf;     end
if (nargin < 4), chans = 1:nChans; end

% Skip past rest of header.
fseek(fd, headerLen + start * nChans * sampleSize, 'bof');

nframe = min(nframe, nFramesTotal - start);   % prevent reading past EOF
nLeft = nFramesTotal - nframe - start;

% Read the data, reshape into sams array.
x = fread(fd, nframe * nChans, 'uint16');
fclose(fd);
sams = reshape(x, nChans, length(x) / nChans).';
sams = sams(:,chans) - 32768;
