function hmark = harufileFromDate_CF2(dt, hIndex, dir)
%harufileFromDate	Given a date/time, find the corresponding Harufile
%
% hmark = harufileFromDate(dt, hIndex)
%    Given a date/time dt encoded as in datenum and an index hIndex from
%    readHarufileDateIndex, determine the file name and the offset
%    (in seconds) within that file.  These are returned as a structure with
%    fields named 
%	hname		a string with the Harufile name, e.g. disk1/0000002.dat
%	time		time in seconds within that file
%	sample		sample number within that file
%	diskno		disk number, for indexing hIndex (numbers start at 1)
%	fileno		file number, for indexing hIndex (numbers start at 0)
%
%    hname is a name like 'disk2/0000000.dat' if multiple disks are involved,
%    or '0000000.dat' if not.  Also handles names like 'datafile.023' and
%    'r0123456.78'.
%
%    dt may be non-scalar (vector or array), in which case hmark is a structure
%    array of the same shape.
%
% See also readHarufileDateIndex, readDetLog.
%
% Dave Mellinger
% David.Mellinger@oregonstate.edu

hmark = struct(...
    'hname',	'',             ...
    'time',	cell(size(dt)), ...
    'sample',	zeros(size(dt)),...
    'diskno',	NaN,            ...
    'fileno',	NaN             );

for i = 1 : prod(size(dt))
  % Find disk, fileno, filestart of latest file starting before or at dt(i).
  earlier = (hIndex.time(:) <= dt(i));
  if (any(earlier))
    [tStart,ix] = max(hIndex.time(earlier));	% get file start-time, index
    ix = ix + sum(earlier(1:ix) == 0);
    tOff = (dt(i) - tStart) * 24*60*60;
    if (tOff < hIndex.dur + 1)		% ensure it's within the file
      [m,n] = ind2sub(size(hIndex.time), ix);
      hmark(i).diskno = n;
      hmark(i).fileno = m-1;  % Harufile nos. start at 0, Matlab indices at 1
      
      % Calculate how long QTech clock says file is, and number of samples.
      [d,f] = nextHaruFile(hmark(i).diskno, hmark(i).fileno, hIndex);
      durQtech = (hIndex.time(f+1, d) - hIndex.time(m,n)) * 24*60*60;
      fname = harufileFromIndex(hmark(i).diskno, hmark(i).fileno, hIndex, dir);
      [dum1,dum2,dum3,dum4,nSamTotal] = haruIn_CF2(fname, 0, 0);
      
      diskdir = sprintf(iff(nCols(hIndex.time) > 1,'disk%d%s',''), ...
	  hmark(i).diskno, filesep);			% string like 'disk1/'
      hmark(i).hname = sprintf('%s%s', diskdir, pathFile(hIndex.hname{m,n}));
      hmark(i).time = (dt(i) - tStart) * 24*60*60;	% in seconds
      hmark(i).sample = round(hmark(i).time / durQtech * nSamTotal);
    end
  end
end
