function hIndex = readHarufileDateIndex(indexfilename)
%readHarufileDateIndex    Extract start-time of Haru files from file_dates file
%
% hIndex = readHarufileDateIndex(indexfilename)
%   Given an indexfilename like 'file_dates_Goa99_N50W135.txt' that contains
%   the starting time of a list of Haru-format files, read the lines in the
%   file one by one and get the start time of each file.  The return
%   value hIndex is a structure:
%       time	  FxD array containing the start time of each file encoded as
%                 in datenum (q.v.); the array is indexed by disk number D and
%                 by F=(file number + 1), since file numbers can start at 0.
%                 The time is the time extracted directly from the Haru file,
%                 which should be GMT.  If a file is missing in the index 
%		  (sometimes sequence numbers are missing), its time is NaN.
%	hname	  FxD cell array: the name of each Harufile, as given in the
%		  index file
%       dur	  (scalar) duration (in s) of each file; same for all files
%       fname	  the input file name (the index file); useful for error 
%                 reporting
%
% See also datenum, harufileDateIndex, goaspermdates, atlminkedates,
%   readDetLog.


fd = fopen(indexfilename);
if (fd < 0)
  error(sprintf('Can''t open file_dates file %s.'), indexfilename); 
end

prevFnum = -1;
disknum = 1;            % Hack!  This doesn't actually look at the disk number
filetimes = zeros(0,1);
hname     = cell(0,1);
maxFnum1 = 0;
while (1)
  % Read next line and check that it's formatted right, not blank, etc.
  ln = fgetl(fd);
  if (length(ln) == 1), if (ln == -1), break; end; end      % EOF?
  if (length(ln) > 20)
    
    % Extract file number.  File names are like datafile.923 or 00000923.dat.
    fnum = NaN;
    if ((strcmp(ln(1:2), '20') | strcmp(ln(1:2), '19')))   % 2000 or 1999 etc.?
      if (strcmpi(ln(end-11 : end-3), 'datafile.'))
        fnum = str2double(ln(end-2 : end));
      elseif (strcmpi(ln(end-3 : end), '.dat'))
        fnum = str2double(ln(end-11 : end-4));
      end
    end                                     % It is not our case...

    % For valid lines, extract date and time, store in filetimes.
    if (~isnan(fnum))
      % If file number decreased, ASSUME (!) it's the next disk.
      if (fnum < prevFnum), disknum = disknum + 1; end
      [t,cnt] = sscanf(ln, '%4d-%3d (%*6c) %d:%d:%g');
      dn = datenum(t(1), 1, t(2), t(3), t(4), t(5));
      while (fnum + 1 > nRows(filetimes))   % ensure filetimes has enough rows
	filetimes(nRows(filetimes) + (1:1000), :) = NaN;
	hname(    nRows(hname)     + (1:1000), :) = {'none'};  %...hname too
      end
      filetimes(:, nCols(filetimes)+1 : disknum) = NaN; % ...and enough cols
      hname(    :, nCols(hname    )+1 : disknum) ={'none'}; %...hname too
      
      filetimes(fnum+1, disknum) = dn; % +1 since datafile numbers start at 000
      hname{fnum+1, disknum} = ln(end-11 : end);
      maxFnum1 = max(maxFnum1, fnum + 1);
      prevFnum = fnum;
    end
  end		% end if (~isnan(fnum))
end		% end while (1)
filetimes = filetimes(1 : maxFnum1, :);
hname     = hname(    1 : maxFnum1, :);

fclose(fd);

% File duration: mean of all durations that are within 2 s of median duration.
d = diff(filetimes(:)) * 24*60*60;		% durations, in seconds
d = d(~isnan(d));
filedur = mean(d(abs(d - median(d)) < 2));

hIndex = struct('time', filetimes, 'dur', filedur, 'hname', {hname}, 'fname', indexfilename);
