function [typ,outfilename,ext,chan,fmt] = soundType(filename)
% typ = soundType(filename)
%
%    Given a file name, figure out what type it is and return it as a text 
%    string.  Often the type is the same as the extension, but not always.
%    Handles multi-row filenames (where the second row specifies a filetype
%    extension or a channel number) correctly.  
%
%    This procedure is used mainly by soundIn.
%
%    Known types:
%
%	Type	Extension(s)
%	----	------------
%	mat	.mat, .m11, .m22		% MATLAB file
%	snd	.snd, .au			% NeXT/Sun audio file
%	aif	.aif				% AIFF file
%	wav	.WAV, .wav			% WAVE file
%	str	.str				% Navy format
%	tim	.TIM				% HyperSignal format
%   haru .DAT               % Haruphone data
%	whole	.adc, .dac, .m11, .m7, .dy22, .dy11, .irc, .macspeech
%	binary	.b1, .b8, .b1_3, .b223, etc.	% 16-bit linear
%	none	everything else
%
%    The file type "whole" is for files that can be read only in their entirety
%    with readSound.  Unfortunately, readSound doesn't exist on Suns.
%
%    Handles two-row filenames correctly.  The optional second and third rows
%    can specify various things about the file.  For example:
%	type=aif	this is an AIFF file (overrides any extension name)
%	chan=3		use channel 3 from the file (default is channel 0)
%	fmt=l		little-endian byte order (default is machine's default)
%
% [typ,filename,ext] = soundType(filename)
%    As above, but also return the extension and filename as separate entities.
%    This is useful when multi-row filenames are a possibility.
%
% [typ,filename,ext,chan] = soundType(filename)
%    As above, but also return the channel number specified on a multi-row
%    filename.  If no channel is specified, chan is [].
%
% [typ,filename,ext,chan,fmt] = soundType(filename)
%    As above, but also return the number format for the file.  This can
%    be 'b' for big-endian (Sun, Mac, HP, etc.) or 'l' for little-endian
%    (Vax,PC).  See fopen for the full list of possibilities.  '' is returned
%    if the user doesn't specify it.  

outfilename = filename(1,:);		% may get changed below
ext = pathExt(outfilename);		% may get changed below
chan = [];				% may get changed below
fmt = '';

% Process the multi-row filename items.
for i = 2:nRows(filename)
  line = deblank(filename(i,:));
  
  if (strncmp(line, 'type', 4))		% check 'type'
    ext = deblank(line(6:length(line)));
    if (ext(1) == '.') 
      ext = ext(2:length(ext)); 
    end
  elseif (strncmp(line, 'chan', 4))	% check 'chan'
    x = line(6:length(line));		% do not combine lines (MATLAB 4.2c bug)
    chan = sscanf(x, '%d');
  elseif (strncmp(line, 'fmt', 3))	% check 'fmt'
    fmt = deblank(line(5:length(line)));
  else
    error(['Unknown multi-row filename item: ', line])
  end
  
  % If this was the longer row, deblank the filename.
  if (nCols(line) == nCols(outfilename))
    outfilename = deblank(outfilename);
  end
end

ext = lower(ext);       % everything turns to the 'Lower case'

% Is it a binary file extension?
if (length(ext) >= 2)
  if (strcmp(ext(1), 'b'))
    e = strsubst(ext, '_', '.');
    [a,count,err,nxt] = sscanf(e(2:length(e)), '%g');
    if (nxt == length(e))
      typ = 'binary';
      return
    end
  end
end

if (  strcmp(ext, 'mat') | ...			% MATLAB file
      strcmp(ext, 'm22') | ...
      strcmp(ext, 'm11'))
  typ = 'mat';
  
elseif (strcmp(ext, 'snd') | ...		% Sun/NeXT sound file
        strcmp(ext, 'au'))
  typ = 'snd';
  
elseif (strcmp(ext, 'str')),			% Navy file
  typ = 'str';
  
elseif (strcmp(lower(ext), 'tim')),		% HyperSignal file
  typ = 'tim';
  
elseif (strcmp(ext, 'aif') | strcmp(ext, 'aiff'))	% AIFF file
  typ = 'aif';
  
elseif (strcmp(lower(ext), 'wav'))		% WAV file
  typ = 'wav';
  
elseif (strcmp(ext, 'adc')       | ...		% must read whole file at once
        strcmp(ext, 'dac')       | ...
	strcmp(ext, 'wav')       | ...
	strcmp(ext, 'm11')       | ...
	strcmp(ext, 'm7')        | ...
        strcmp(ext, 'dy22')      | ...
        strcmp(ext, 'dy11')      | ...
        strcmp(ext, 'irc')       | ...
	strcmp(ext, 'macspeech'))
  typ = 'whole';
  
%% Won Sang Lee puts this lines to recognize haruphone data, Jul 1, 2007 %%

elseif (strcmp(ext, 'dat')),            % lower(ext): 'DAT' -> 'dat'
    typ = 'haru';
    
%%
    
%elseif (isempty(ext))
  % Special case for Harold Mills -- extensionless files are 22050 Hz binary.
  %typ = 'harold';

else						% unknown or non-soundfile
  typ = 'none';

end
